"""
This application provides GUI control for an USB generic HID device
built form a NUCLEO-F446 board: Firmware and description is 
available here: https://github.com/cspista/mbed6_usbHID_PWM 

The concept and the report format is based on the Cypress
AN82072 Application Note

Tested with PySide 6.3.2, PyWinUsb 0.4.2, Python 3.10
"""
import sys
from PySide6 import QtWidgets
from ui_HIDTest import Ui_Dialog
import pywinusb.hid as hid

ix,sw1,adc = (0,0,0)

class MainWindow(Ui_Dialog,hid.HidPnPWindowMixin ):
    # Class properties - feel free to test
    vID = 0x1234
    pID = 0x0006
    my_filter = hid.HidDeviceFilter(vendor_id = vID, product_id = pID)
    led_state = 1
    pwm_data =  10

    def __init__(self):
        super(MainWindow, self).__init__()       
        self.setupUi(self)
        hid.HidPnPWindowMixin.__init__(self, self.winId())
        self.label_8.setStyleSheet("color: red")
        self.pushButton_2.setStyleSheet("color: white; background-color: red")
        self.pushButton_2.setText("Off")
        self.pushButton.clicked.connect(self.on_update)
        self.pushButton_3.clicked.connect(self.on_set)
        self.checkBox.stateChanged.connect(self.on_check)

        # kick the pnp engine
        self.device = None        #no hid device... yet
        self.on_hid_pnp()

    def sample_handler(self,data):
        global sw1
        global adc
        global ix
        sw1 = sw1 + data[1]
        adc = adc + data[4]*256 + data[5]
        ix = ix + 1
        if ix == 100:
            # self.lineEdit.setText("0x{0:04X}".format(adc//100))
            self.lineEdit.setText("{0:.3f} V".format(adc*3.3/6553600))
            if sw1>50:
                self.pushButton_2.setStyleSheet("color: white; background-color: green")      
                self.pushButton_2.setText("ON") 
            else:
               self.pushButton_2.setStyleSheet("color: white; background-color: red")      
               self.pushButton_2.setText("Off")
            ix,sw1,adc = (0,0,0)

    def send_command(self,led1,led2):    # Send the message to the Mbed board
        # The first byte is the report ID which must be 0
        buffer = [0 for i in range(9)]  # Array of 9 elements  
        buffer[1] = led1
        buffer[2] = led2
        #-- print(len(buffer),": ", buffer)
        if self.device is not None and self.device.is_plugged():
            out_report = self.device.find_output_reports()
            out_report[0].set_raw_data(buffer)
            out_report[0].send() 

    def on_update(self):
        self.pwm_data = int(self.spinBox.value())
        self.send_command(self.led_state,self.pwm_data)

    def on_set(self):
        self.vID = int(self.lineEdit_2.text(), 16)
        self.pID = int(self.lineEdit_3.text(), 16)   
        self.my_filter = hid.HidDeviceFilter(vendor_id = self.vID, product_id = self.pID)
        self.test_for_connection()

    def on_check(self,state):
        if state:
           self.led_state = 1
        else:
           self.led_state = 0
        self.send_command(self.led_state,self.pwm_data) 

    def on_hid_pnp(self, hid_event = None):
        """This function will be called on per class event changes, so we need
        to test if our device has being connected or is just gone"""
        # keep old reference for UI updates
        old_device = self.device
        '''
        if hid_event:
            print("Hey, a hid device just %s!" % hid_event)
        '''
        if hid_event == "connected":
            # test if our device is available
            if self.device:
                # see, at this point we could detect multiple devices!
                # but... we only want just one
                pass
            else:
                self.test_for_connection()
        elif hid_event == "disconnected":
            # the hid object is automatically closed on disconnection we just
            # test if still is plugged (important as the object might be
            # closing)
            if self.device and not self.device.is_plugged():
                self.device = None
                self.label_8.setText("Disconnected")
                self.label_8.setStyleSheet("color: red")  
        else:
            # poll for devices
            self.test_for_connection()

        if old_device != self.device:
            # update ui
            pass

    def test_for_connection(self):
        all_items =  self.my_filter.get_devices()
        if all_items:
            # at this point, what we decided to be a valid hid target is
            # already plugged
            if len(all_items) == 1:
                # this is easy, we only have a single hid device
                self.device = all_items[0]
            else:
                # at this point you might have multiple scenarios
                grouped_items = MyFrame.my_hid_target.get_devices_by_parent()
                print("%d devices now connected" % len(grouped_items))
                if len(grouped_items) > 1:
                    # 1) Really you have multiple devices connected so, make
                    # your rules, how do you help your user to handle multiple
                    # devices?
                    # maybe you here will find out wich is the new device, and
                    # tag this device so is easily identified (i.e. the WiiMote
                    # uses LEDs), or just your GUI shows some arbitrary
                    # identification for the user (device 2 connected)
                    pass
                else:
                    # 2) We have a single physical device, but the descriptors
                    # might might cause the OS to report is as multiple devices
                    # (collections maybe) so, what would be your target device?
                    # if you designed the device firmware, you already know the
                    # answer...  otherwise one approach might be to browse the
                    # hid usages for a particular target...  anyway, this could
                    # be complex, especially handling multiple physical devices
                    # that are reported as multiple hid paths (objects)...
                    # so...  I recommend you creating a proxy class that is
                    # able to handle all your 'per parent id' grouped devices,
                    # (like a single .open() able to handle your buch of
                    # HidDevice() items
                    pass
                # but... we just arbitrarly select the first hid object path
                # (how creative!)
                self.device = all_items[0]

        if all_items and self.device:
            self.device.open()
            self.device.set_raw_data_handler(self.sample_handler)
            self.label_8.setText("Connected")
            self.label_8.setStyleSheet("color: green") 
        else:
            if self.device is not None:
                self.device.set_raw_data_handler(None)
                self.device.close() 
                self.device = None
            self.label_8.setText("Disconnected")
            self.label_8.setStyleSheet("color: red")  



app = QtWidgets.QApplication([])
window = MainWindow()
window.show()
app.exec()
